#!/usr/bin/python3

import json
import os
import tkinter as tk
from tkinter import simpledialog, messagebox
import re
import pwd
import sys
import subprocess
from getpass import getpass

def load_translations(file_path):
    """Carica le traduzioni da un file JSON."""
    if not os.path.exists(file_path):
        raise FileNotFoundError(f"File di traduzione non trovato: {file_path}")
    with open(file_path, 'r', encoding='utf-8') as f:
        return json.load(f)

# Funzione per tradurre i messaggi
def translate(key):
    """Ottiene una traduzione per una chiave specifica."""
    return locale_translations.get(key, key)

def get_user_home():
    """Ottiene il percorso della home directory dell'utente originale"""
    sudo_user = os.environ.get("SUDO_USER")
    if sudo_user:
        return pwd.getpwnam(sudo_user).pw_dir
    else:
        return os.path.expanduser("~")

def get_locale_from_dmrc():
    """Estrae il locale dal file ~/.dmrc"""
    if not os.path.exists(dmrc_path):
        return "en_US"  # Default fallback

    try:
        with open(dmrc_path, "r") as file:
            for line in file:
                match = re.search(r"Language=([a-zA-Z_]+)\.(?:UTF-8|utf8)", line)
                if match:
                    return match.group(1)
    except Exception as e:
        print(f"Errore durante la lettura di {dmrc_path}: {e}")

    return "en_US"  # Default fallback

def check_sudo_privileges():
    """Verifica se l'utente ha privilegi sudo."""
    try:
        result = subprocess.run(['groups'], stdout=subprocess.PIPE, text=True)
        if 'sudo' in result.stdout:
            return True
    except Exception as e:
        messagebox.showerror(translate("error_title"), translate("sudo_check_error").format(error=e))
    return False

def prompt_password_and_relaunch():
    """Mostra una finestra di dialogo per richiedere la password e rilancia lo script come sudo."""

    # Chiedi la password con una finestra di dialogo
    password = simpledialog.askstring(
        translate("auth_required_title"),
        translate("enter_password"),
        show="*"
    )

    if password is None:  # Se l'utente chiude la finestra o annulla
        messagebox.showinfo(translate("operation_cancelled_title"), translate("operation_cancelled_message"))
        sys.exit(1)

    try:
        # Rilancia lo script con sudo
        result = subprocess.run(
            ['sudo', '-S', sys.executable] + sys.argv,
            input=password + '\n',
            text=True
        )
        if result.returncode != 0:
            messagebox.showerror(translate("error_title"), translate("auth_failed"))
            sys.exit(1)
    except Exception as e:
        messagebox.showerror(translate("error_title"), translate("auth_error").format(error=e))
        sys.exit(1)

def backup_file(file_path):
    """Crea un backup del file."""
    backup_path = f"{file_path}.bak"
    if not os.path.exists(backup_path):
        os.system(f"cp {file_path} {backup_path}")

def is_proxy_present(file_path, proxy_line):
    """Controlla se la riga di proxy è già presente nel file."""
    try:
        if not os.path.exists(file_path):
            return False
        with open(file_path, 'r') as f:
            content = f.read()
        return proxy_line.strip() in content
    except Exception as e:
        messagebox.showerror(
            locale_translations["title"],
            locale_translations["error_message"].format(error=e)
        )
        return False

def configure_proxy():
    """Aggiunge le righe di configurazione proxy ai file."""
    try:
        proxy_added = False  # Flag per tracciare se almeno un file è stato modificato

        for file_path, proxy_line in FILES.items():
            if is_proxy_present(file_path, proxy_line):
                messagebox.showinfo(
                    locale_translations["title"],
                    locale_translations["proxy_already_present"].format(file_path=file_path)
                )
                continue
            
            # Backup e scrittura del proxy
            backup_file(file_path)
            with open(file_path, 'a') as f:
                f.write(proxy_line)
            proxy_added = True  # Indica che almeno un file è stato modificato

        if proxy_added:
            # Mostra il messaggio di successo
            messagebox.showinfo(
                locale_translations["title"],
                locale_translations["success_message"]
            )
            # Mostra il messaggio di riavvio della sessione
            messagebox.showinfo(
                locale_translations["title"],
                locale_translations["restart_session_message"]
            )

    except Exception as e:
        messagebox.showerror(
            locale_translations["title"],
            locale_translations["error_message"].format(error=e)
        )

def remove_proxy():
    """Rimuove le righe di configurazione proxy dai file."""
    try:
        for file_path, proxy_lines in FILES.items():
            if os.path.exists(file_path):
                with open(file_path, 'r') as f:
                    lines = f.readlines()
                
                proxy_lines_list = proxy_lines.strip().split('\n')
                with open(file_path, 'w') as f:
                    f.writelines([line for line in lines if line.strip() not in proxy_lines_list])
        
        messagebox.showinfo(
            locale_translations["title"],
            locale_translations["remove_message"]
        )
    except Exception as e:
        messagebox.showerror(
            locale_translations["title"],
            locale_translations["error_message"].format(error=e)
        )

# Percorso del file di traduzione
TRANSLATION_FILE = "/usr/share/fuss-proxy/translations/translations.json"

# Carica traduzioni
try:
    translations = load_translations(TRANSLATION_FILE)
except FileNotFoundError as e:
    print(e)
    translations = {"en_US": {}}  # Fallback

user_home = get_user_home()

dmrc_path = os.path.join(user_home, ".dmrc")

system_locale = get_locale_from_dmrc()

print(f"Locale rilevato: {system_locale}")

locale_translations = translations.get(system_locale, translations["en_US"])

# Verifica se l'utente è root
if os.geteuid() != 0:
    if check_sudo_privileges():
        prompt_password_and_relaunch()
    else:
        messagebox.showerror(translate("error_title"), translate("sudo_required"))
        sys.exit(1)

# Da qui in poi, il codice principale dell'applicazione

FILES = {
    "/etc/apt/apt.conf.d/90aptcacher": (
        "# BEGIN ADDED BY FUSS_PROXY\n"
        "# using server apt-cacher-ng proxy\n"
        'Acquire::http::Proxy "http://proxy:3142";\n'
        'Acquire::https::Proxy "http://proxy:3142";\n'
        "# END ADDED BY FUSS_PROXY\n"
    ),
    "/etc/environment": (
        "# BEGIN ADDED BY FUSS_PROXY\n"        
        'http_proxy=http://proxy:8080\n'
        'https_proxy=http://proxy:8080\n'
        'ftp_proxy=http://proxy:8080\n'
        'no_proxy=localhost,127.0.0.1,127.0.0.0,192.168.0.0/24\n'
        'NO_PROXY=localhost,127.0.0.1,127.0.0.0,192.168.0.0/24\n'
        "# END ADDED BY FUSS_PROXY\n"
    ),
    "/etc/wgetrc": (
        "# BEGIN ADDED BY FUSS_PROXY\n"
        'http_proxy = http://proxy:8080\n'
        'https_proxy = http://proxy:8080\n'
        'use_proxy = on\n'
        "# END ADDED BY FUSS_PROXY\n"
    )
}

# Evita di creare più istanze della finestra principale
root = tk.Tk()
root.withdraw()  # Nasconde la finestra principale

# Interfaccia grafica
app = tk.Tk()
app.title("Proxy")

screen_width = app.winfo_screenwidth()
screen_height = app.winfo_screenheight()

window_width = 400
window_height = 200

x_position = (screen_width // 2) - (window_width // 2)
y_position = (screen_height // 2) - (window_height // 2)

app.geometry(f"{window_width}x{window_height}+{x_position}+{y_position}")

#title_label = tk.Label(app, text=locale_translations["title"], font=("Helvetica", 16), bg="lightblue", width=window_width)
title_label = tk.Label(
    app,
    text=locale_translations["title"],
    font=("Helvetica", 16),
    bg="#32CD32",  # Verde personalizzato
    width=window_width,
    height=2,  # Aumenta l'altezza del pannello
    anchor="center"  # Centra il testo verticalmente e orizzontalmente
)
title_label.pack(fill=tk.X)

spacer = tk.Label(app, text="")
spacer.pack(pady=5)

configure_button = tk.Button(app, text=locale_translations["configure_button"], command=configure_proxy, width=20)
configure_button.pack(pady=10)

remove_button = tk.Button(app, text=locale_translations["remove_button"], command=remove_proxy, width=20)
remove_button.pack(pady=10)

app.mainloop()

